/******************************************************************************/
/*                                                                            */
/*    The Multilevel Grid File (MLGF)                                         */
/*    Version 4.0                                                             */
/*                                                                            */
/*    Developed by Professor Kyu-Young Whang et al. (1994-2016)               */
/*                                                                            */
/*    Advanced Information Technology Research Center (AITrc)                 */
/*    Korea Advanced Institute of Science and Technology (KAIST)              */
/*                                                                            */
/*    e-mail: kywhang@gmail.com                                               */
/*                                                                            */
/*    Bibliography:                                                           */
/*    [1] Whang, K. and Krishnamurthy, R., "The Multilevel Grid File - A      */
/*        Dynamic Hierarchical Multidimensional File Structure," In Proc. 2nd */
/*        Int'l Conf. on Database Systems for Advanced Applications, pp.      */
/*        449-459, 1991.                                                      */
/*    [2] Whang, K., Kim, S., and Wiederhold, G., "Dynamic Maintenance of     */
/*        Data Distribution for Selectivity Estimation," The VLDB Journal,    */
/*        Vol. 3, No. 1, pp. 29-51, 1994.                                     */
/*    [3] Lee, J., Lee, Y., and Whang, K., "A Region Splitting Strategy for   */
/*        Physical Database Design of Multidimensional File Organizations,"   */
/*        In Proc. 23rd Int'l Conf. on Very Large Data Bases, pp. 416-425,    */
/*        1997.                                                               */
/*    [4] Song, J., Whang, K., Lee, Y., Lee, M., and Kim, S., "Spatial Join   */
/*        Processing Using Corner Transformation, IEEE Transactions on        */
/*        Knowledge and Data Engineering (TKDE), Vol. 11, No. 4, pp. 688-695, */
/*        1999.                                                               */
/*    [5] Song, J. et al., "The Clustering Property of Corner Transformation  */
/*        for Spatial Database Applications," Information and Software        */
/*        Technology, Vol. 44, No. 7, pp. 419-429, 2002.                      */
/*    [6] Lee, M., Whang, K., Han, W., and Song, I., "Transform-Space View:   */
/*        Performing Spatial Join in the Transform Space Using Original-Space */
/*        Indexes," IEEE Transactions on Knowledge and Data Engineering       */
/*        (TKDE), Vol 18, No. 2, pp. 245-260, 2006.                           */
/*    [7] Dai, H., Whang, K., and Su, H., "Locality of Corner Transformation  */
/*        for Multidimensional Spatial Access Methods," Electronic Notes in   */
/*        Theoretical Computer Science, Vol. 212, pp. 133-148, 2008.          */
/*                                                                            */
/******************************************************************************/

#include <BfM.h>

#define BUFT(i) (BfM_BufferTable[(i)])

void BfM_dump_buffertable(void)
{
    int i;

    printf("\n\t|==================================================|\n");
    printf("\t|                 Buffer Table                     |\n");
    printf("\t|-------------+-------------+-------------+--------|\n");
    printf("\t|%10s   |%10s   |%10s   |  bits  |\n", "SegID", "pageid", "fixed");
    printf("\t|-------------+-------------+-------------+--------|\n");
    for(i = 0; i < BfM_MaxBufs; i++) 
        printf("\t|%10d   |%10d   |%10d   |  0x%-2x  |\n", BUFT(i).Key.SegID, 
                BUFT(i).Key.pageid, BUFT(i).fixed, (int)BUFT(i).bits);
    printf("\t|==================================================|\n");
}

void BfM_dump_hashtable(void)
{
    int i;
    int j;

    printf("\n\t|=======================================================================================|\n");
    printf("\t|                                        Hash Table                                     |\n");
    printf("\t|=======|=======|=======|=======|=======|=======|=======|=======|=======|=======|=======|\n");
    printf("\t|       |    0  |    1  |    2  |    3  |    4  |    5  |    6  |    7  |    8  |    9  |\n");
    printf("\t|=======|=======|=======|=======|=======|=======|=======|=======|=======|=======|=======|\n");
    for(i = 0; i <= (int)(BfM_HashTableSize/10);i++) {
        printf("\t|%5d  |", i);
        for(j = 0; j < 10; j++) 
            if((i*10+j) < BfM_HashTableSize)
                printf("%5d  |", BfM_HashTable[i*10+j]);
            else
                printf("       |");
        if(i < (int)(BfM_HashTableSize/10))
            printf("\n\t|-------|-------|-------|-------|-------|-------|-------|-------|-------|-------|-------|\n");
    }
    printf("\n\t|=======|=======|=======|=======|=======|=======|=======|=======|=======|=======|=======|\n");

}
