/******************************************************************************/
/*                                                                            */
/*    The Multilevel Grid File (MLGF)                                         */
/*    Version 4.0                                                             */
/*                                                                            */
/*    Developed by Professor Kyu-Young Whang et al. (1994-2016)               */
/*                                                                            */
/*    Advanced Information Technology Research Center (AITrc)                 */
/*    Korea Advanced Institute of Science and Technology (KAIST)              */
/*                                                                            */
/*    e-mail: kywhang@gmail.com                                               */
/*                                                                            */
/*    Bibliography:                                                           */
/*    [1] Whang, K. and Krishnamurthy, R., "The Multilevel Grid File - A      */
/*        Dynamic Hierarchical Multidimensional File Structure," In Proc. 2nd */
/*        Int'l Conf. on Database Systems for Advanced Applications, pp.      */
/*        449-459, 1991.                                                      */
/*    [2] Whang, K., Kim, S., and Wiederhold, G., "Dynamic Maintenance of     */
/*        Data Distribution for Selectivity Estimation," The VLDB Journal,    */
/*        Vol. 3, No. 1, pp. 29-51, 1994.                                     */
/*    [3] Lee, J., Lee, Y., and Whang, K., "A Region Splitting Strategy for   */
/*        Physical Database Design of Multidimensional File Organizations,"   */
/*        In Proc. 23rd Int'l Conf. on Very Large Data Bases, pp. 416-425,    */
/*        1997.                                                               */
/*    [4] Song, J., Whang, K., Lee, Y., Lee, M., and Kim, S., "Spatial Join   */
/*        Processing Using Corner Transformation, IEEE Transactions on        */
/*        Knowledge and Data Engineering (TKDE), Vol. 11, No. 4, pp. 688-695, */
/*        1999.                                                               */
/*    [5] Song, J. et al., "The Clustering Property of Corner Transformation  */
/*        for Spatial Database Applications," Information and Software        */
/*        Technology, Vol. 44, No. 7, pp. 419-429, 2002.                      */
/*    [6] Lee, M., Whang, K., Han, W., and Song, I., "Transform-Space View:   */
/*        Performing Spatial Join in the Transform Space Using Original-Space */
/*        Indexes," IEEE Transactions on Knowledge and Data Engineering       */
/*        (TKDE), Vol 18, No. 2, pp. 245-260, 2006.                           */
/*    [7] Dai, H., Whang, K., and Su, H., "Locality of Corner Transformation  */
/*        for Multidimensional Spatial Access Methods," Electronic Notes in   */
/*        Theoretical Computer Science, Vol. 212, pp. 133-148, 2008.          */
/*                                                                            */
/******************************************************************************/

/*
 * Module: mlgf_FindBuddyEntry.c
 *
 * Description:
 *  Find a buddy entry of the given entry.
 *
 * Exports:
 *  Boolean mlgf_FindBuddyEntry(mlgf_DirectoryPage*, One, mlgf_DirectoryEntry*,
 *                              MLGF_EntryIndex*, One*)
 *
 * Returns:
 *  TRUE if a buddy entry is found
 *  FALSE otherwise
 */


#include "common.h"


Boolean mlgf_FindBuddyEntry(
    mlgf_DirectoryPage  *dirPage,       /* IN a directory page */
    One                 nKeys,          /* IN number of keys in MLGF */
    mlgf_DirectoryEntry *entry,         /* IN we find buddy entry of this 'entry' */
    Two                 *buddyEntryNo,  /* OUT entry no of buddy entry */
    One                 *buddyKey)      /* OUT buddy key no */
{
    Two                 i;              /* index variable */
    Two                 entryLen;       /* length of a directory entry */
    mlgf_DirectoryEntry *otherEntry;    /* entries in directory page */


    /* Calculate the length of a directory entry. */
    entryLen = MLGF_DIRENTRY_LENGTH(nKeys);

    /* for all the entries */
    for (i = 0; i < dirPage->hdr.nEntries; i++) {
	otherEntry = MLGF_ITH_DIRENTRY(dirPage, i, entryLen);

	/* do buddy region test */
	if (mlgf_BuddyTest(nKeys, entry, otherEntry, buddyKey) == TRUE) {
	    /* return the entry no of buddy entry */
	    *buddyEntryNo = i;

	    return(TRUE);
	}
    }

    return(FALSE);

} /* mlgf_FindBuddyEntry() */
