/******************************************************************************/
/*                                                                            */
/*    The Multilevel Grid File (MLGF)                                         */
/*    Version 4.0                                                             */
/*                                                                            */
/*    Developed by Professor Kyu-Young Whang et al. (1994-2016)               */
/*                                                                            */
/*    Advanced Information Technology Research Center (AITrc)                 */
/*    Korea Advanced Institute of Science and Technology (KAIST)              */
/*                                                                            */
/*    e-mail: kywhang@gmail.com                                               */
/*                                                                            */
/*    Bibliography:                                                           */
/*    [1] Whang, K. and Krishnamurthy, R., "The Multilevel Grid File - A      */
/*        Dynamic Hierarchical Multidimensional File Structure," In Proc. 2nd */
/*        Int'l Conf. on Database Systems for Advanced Applications, pp.      */
/*        449-459, 1991.                                                      */
/*    [2] Whang, K., Kim, S., and Wiederhold, G., "Dynamic Maintenance of     */
/*        Data Distribution for Selectivity Estimation," The VLDB Journal,    */
/*        Vol. 3, No. 1, pp. 29-51, 1994.                                     */
/*    [3] Lee, J., Lee, Y., and Whang, K., "A Region Splitting Strategy for   */
/*        Physical Database Design of Multidimensional File Organizations,"   */
/*        In Proc. 23rd Int'l Conf. on Very Large Data Bases, pp. 416-425,    */
/*        1997.                                                               */
/*    [4] Song, J., Whang, K., Lee, Y., Lee, M., and Kim, S., "Spatial Join   */
/*        Processing Using Corner Transformation, IEEE Transactions on        */
/*        Knowledge and Data Engineering (TKDE), Vol. 11, No. 4, pp. 688-695, */
/*        1999.                                                               */
/*    [5] Song, J. et al., "The Clustering Property of Corner Transformation  */
/*        for Spatial Database Applications," Information and Software        */
/*        Technology, Vol. 44, No. 7, pp. 419-429, 2002.                      */
/*    [6] Lee, M., Whang, K., Han, W., and Song, I., "Transform-Space View:   */
/*        Performing Spatial Join in the Transform Space Using Original-Space */
/*        Indexes," IEEE Transactions on Knowledge and Data Engineering       */
/*        (TKDE), Vol 18, No. 2, pp. 245-260, 2006.                           */
/*    [7] Dai, H., Whang, K., and Su, H., "Locality of Corner Transformation  */
/*        for Multidimensional Spatial Access Methods," Electronic Notes in   */
/*        Theoretical Computer Science, Vol. 212, pp. 133-148, 2008.          */
/*                                                                            */
/******************************************************************************/

/*
 * Module: mlgf_GetOpenFileTableEntry.c
 *
 * Description:
 *  Get an empty slot of the open file table.
 *  Returns the empty slot number.
 *
 * Exports:
 *  Four mlgf_GetOpenFileTableEntry()
 *
 * Returns:
 *  Slot number of an empty slot of the open file table
 *     or
 *  Error code
 *    eMEMORYALLOCERR
 */


#include "common.h"


/* open file table */
OpenFileTableEntry	*openFileTable = NULL;
int	maxEntriesOfOpenFileTable = 0;


Four mlgf_GetOpenFileTableEntry()
{
	int	i, j;
	OpenFileTableEntry	*ptr;
	
	
	if(openFileTable == NULL) {
		openFileTable = (OpenFileTableEntry*)malloc(sizeof(OpenFileTableEntry)*OPENFILETABLE_INCREMENT_SIZE);
		if(openFileTable == NULL) ERR(eMEMORYALLOCERR);
		maxEntriesOfOpenFileTable = OPENFILETABLE_INCREMENT_SIZE;
	    
	    for(i = 0; i < maxEntriesOfOpenFileTable; i++)
			OPENFILE_INO(i) = NIL;
	}
	
	for(i = 0; i < maxEntriesOfOpenFileTable; i++)
	    if(OPENFILE_INO(i) == NIL) break;
	
	/* Is there an empty slot? */
	if(i == maxEntriesOfOpenFileTable) {
	    ptr = (OpenFileTableEntry*)malloc(sizeof(OpenFileTableEntry)*(maxEntriesOfOpenFileTable+OPENFILETABLE_INCREMENT_SIZE));
	    if(ptr == NULL) ERR(eMEMORYALLOCERR);
	    maxEntriesOfOpenFileTable += OPENFILETABLE_INCREMENT_SIZE;
	    
	    for(j = i; j < maxEntriesOfOpenFileTable; j++)
			OPENFILE_INO(j) = NIL;
	}
	
	return(i);    
} /* mlgf_GetOpenFileTableEntry() */
