/******************************************************************************/
/*                                                                            */
/*    The Multilevel Grid File (MLGF)                                         */
/*    Version 4.0                                                             */
/*                                                                            */
/*    Developed by Professor Kyu-Young Whang et al. (1994-2016)               */
/*                                                                            */
/*    Advanced Information Technology Research Center (AITrc)                 */
/*    Korea Advanced Institute of Science and Technology (KAIST)              */
/*                                                                            */
/*    e-mail: kywhang@gmail.com                                               */
/*                                                                            */
/*    Bibliography:                                                           */
/*    [1] Whang, K. and Krishnamurthy, R., "The Multilevel Grid File - A      */
/*        Dynamic Hierarchical Multidimensional File Structure," In Proc. 2nd */
/*        Int'l Conf. on Database Systems for Advanced Applications, pp.      */
/*        449-459, 1991.                                                      */
/*    [2] Whang, K., Kim, S., and Wiederhold, G., "Dynamic Maintenance of     */
/*        Data Distribution for Selectivity Estimation," The VLDB Journal,    */
/*        Vol. 3, No. 1, pp. 29-51, 1994.                                     */
/*    [3] Lee, J., Lee, Y., and Whang, K., "A Region Splitting Strategy for   */
/*        Physical Database Design of Multidimensional File Organizations,"   */
/*        In Proc. 23rd Int'l Conf. on Very Large Data Bases, pp. 416-425,    */
/*        1997.                                                               */
/*    [4] Song, J., Whang, K., Lee, Y., Lee, M., and Kim, S., "Spatial Join   */
/*        Processing Using Corner Transformation, IEEE Transactions on        */
/*        Knowledge and Data Engineering (TKDE), Vol. 11, No. 4, pp. 688-695, */
/*        1999.                                                               */
/*    [5] Song, J. et al., "The Clustering Property of Corner Transformation  */
/*        for Spatial Database Applications," Information and Software        */
/*        Technology, Vol. 44, No. 7, pp. 419-429, 2002.                      */
/*    [6] Lee, M., Whang, K., Han, W., and Song, I., "Transform-Space View:   */
/*        Performing Spatial Join in the Transform Space Using Original-Space */
/*        Indexes," IEEE Transactions on Knowledge and Data Engineering       */
/*        (TKDE), Vol 18, No. 2, pp. 245-260, 2006.                           */
/*    [7] Dai, H., Whang, K., and Su, H., "Locality of Corner Transformation  */
/*        for Multidimensional Spatial Access Methods," Electronic Notes in   */
/*        Theoretical Computer Science, Vol. 212, pp. 133-148, 2008.          */
/*                                                                            */
/******************************************************************************/

/*
 * Module: mlgf_ObjectArray.c
 *
 * Description:
 *  Routines for the object array.
 *  The objects in one array have the same
 *  keys.
 *
 * Exports:
 *  Boolean mlgf_SearchObjectArray(char*, Object*, Four, Two*, Four*)
 *
 * Returns:
 *  TRUE if we find the given object
 *  FALSE otherwise
 */


#include "common.h"


/*
 * Function: Boolean mlgf_SearchObjectArray(char*, Object*, Four, Two*, Four*)
 *
 * Description:
 *  Search the object array for an entry whose ObjectID is equal to the given
 *  ObjectID. If found, returns its element No. Otherwise, the entry No whose
 *  ObjectID is smallest but not less than the given ObjectID.
 *
 * Returns:
 *  TRUE if we find the given object
 *  FALSE otherwise
 */
Boolean mlgf_SearchObjectArray(
    char                        *array,                 /* IN starting point of array */
	Object						*obj,					/* IN search for this object */
    Four                        nObjects,               /* IN number of ojbects in the array */
    Two                         *elemNo,                /* OUT element number of the object found */
	Four						*elemOffset,			/* OUT element offset of the object found */
	Two			mlgfd)									/* IN MLGF file descriptor */
{
	Four						i;						/* idx */
    Four                        cmp;                    /* result of comparison */
	Four						currentOffset;			/* offset of current object */
	Two							curObjLen;				/* length of the current object */
	Object						*objPtr;				/* pointer to an object */
	Two							dupType;				/* duplication of type */


	currentOffset = 0;
	for (i=0; i<nObjects; i++)
	{
		objPtr = (Object*)&array[currentOffset];

		 curObjLen = mlgf_leafEntryObjectItemLen(mlgfd,objPtr);

		 dupType = mlgf_isSameObject(mlgfd,obj,objPtr);
		if(dupType != FALSE)
		{
			*elemNo = i;
			*elemOffset = currentOffset;

			return dupType;
		}

		/* Points to the next object */
		currentOffset += curObjLen;
	}

	*elemNo = i;
	*elemOffset = currentOffset;

	return(FALSE);

} /* mlgf_binarySearchObjectArray*() */

